@@ -19,6 +19,7 @@ import numpy as np
 import torch
 import torch.nn as nn
 from opacus.utils.tensor_utils import unfold2d, unfold3d
+from opt_einsum import contract
 
 from .utils import register_grad_sampler
 
@@ -66,22 +67,25 @@ def compute_conv_grad_sample(
             dilation=layer.dilation,
         )
     backprops = backprops.reshape(n, -1, activations.shape[-1])
-    # n=batch_sz; o=num_out_channels; p=(num_in_channels/groups)*kernel_sz
-    grad_sample = torch.einsum("noq,npq->nop", backprops, activations)
-    # rearrange the above tensor and extract diagonals.
-    grad_sample = grad_sample.view(
-        n,
-        layer.groups,
-        -1,
-        layer.groups,
-        int(layer.in_channels / layer.groups),
-        np.prod(layer.kernel_size),
-    )
-    grad_sample = torch.einsum("ngrg...->ngr...", grad_sample).contiguous()
-    shape = [n] + list(layer.weight.shape)
 
-    ret = {layer.weight: grad_sample.view(shape)}
-    if layer.bias is not None:
+    ret = {}
+    if layer.weight.requires_grad:
+        # n=batch_sz; o=num_out_channels; p=(num_in_channels/groups)*kernel_sz
+        grad_sample = contract("noq,npq->nop", backprops, activations)
+        # rearrange the above tensor and extract diagonals.
+        grad_sample = grad_sample.view(
+            n,
+            layer.groups,
+            -1,
+            layer.groups,
+            int(layer.in_channels / layer.groups),
+            np.prod(layer.kernel_size),
+        )
+        grad_sample = contract("ngrg...->ngr...", grad_sample).contiguous()
+        shape = [n] + list(layer.weight.shape)
+        ret[layer.weight] = grad_sample.view(shape)
+
+    if layer.bias is not None and layer.bias.requires_grad:
         ret[layer.bias] = torch.sum(backprops, dim=2)
 
     return ret
