@@ -19,6 +19,7 @@ from typing import Dict
 import torch
 import torch.nn as nn
 import torch.nn.functional as F
+from opt_einsum import contract
 
 from .utils import register_grad_sampler
 
@@ -37,8 +38,10 @@ def compute_group_norm_grad_sample(
         activations: Activations
         backprops: Backpropagations
     """
-    gs = F.group_norm(activations, layer.num_groups, eps=layer.eps) * backprops
-    ret = {layer.weight: torch.einsum("ni...->ni", gs)}
-    if layer.bias is not None:
-        ret[layer.bias] = torch.einsum("ni...->ni", backprops)
+    ret = {}
+    if layer.weight.requires_grad:
+        gs = F.group_norm(activations, layer.num_groups, eps=layer.eps) * backprops
+        ret[layer.weight] = contract("ni...->ni", gs)
+    if layer.bias is not None and layer.bias.requires_grad:
+        ret[layer.bias] = contract("ni...->ni", backprops)
     return ret
