@@ -19,6 +19,7 @@ from typing import Callable, List, Optional, Union
 
 import torch
 from opacus.optimizers.utils import params
+from opt_einsum import contract
 from torch import nn
 from torch.optim import Optimizer
 
@@ -280,9 +281,9 @@ class DPOptimizer(Optimizer):
         self.generator = generator
         self.secure_mode = secure_mode
 
-        self.param_groups = optimizer.param_groups
+        self.param_groups = self.original_optimizer.param_groups
         self.defaults = self.original_optimizer.defaults
-        self.state = optimizer.state
+        self.state = self.original_optimizer.state
         self._step_skip_queue = []
         self._is_last_step_skipped = False
 
@@ -404,7 +405,7 @@ class DPOptimizer(Optimizer):
             _check_processed_flag(p.grad_sample)
 
             grad_sample = _get_flat_grad_sample(p)
-            grad = torch.einsum("i,i...", per_sample_clip_factor, grad_sample)
+            grad = contract("i,i...", per_sample_clip_factor, grad_sample)
 
             if p.summed_grad is not None:
                 p.summed_grad += grad
