@@ -12,8 +12,9 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+import os
 import warnings
-from typing import List, Optional, Tuple, Union
+from typing import IO, Any, BinaryIO, Dict, List, Optional, Tuple, Union
 
 import torch
 from opacus.accountants import create_accountant
@@ -22,6 +23,8 @@ from opacus.data_loader import DPDataLoader, switch_generator
 from opacus.distributed import DifferentiallyPrivateDistributedDataParallel as DPDDP
 from opacus.grad_sample.grad_sample_module import GradSampleModule
 from opacus.optimizers import DPOptimizer, get_optimizer_class
+from opacus.scheduler import _NoiseScheduler
+from opacus.utils.module_utils import trainable_parameters
 from opacus.validators.module_validator import ModuleValidator
 from torch import nn, optim
 from torch.nn.parallel import DistributedDataParallel as DDP
@@ -56,7 +59,7 @@ def forbid_accumulation_hook(
     if not module.training:
         return
 
-    for p in module.parameters():
+    for _, p in trainable_parameters(module):
         if p.grad_sample is not None:
             if isinstance(p.grad_sample, torch.Tensor):
                 accumulated_iterations = 1
@@ -334,12 +337,12 @@ class PrivacyEngine:
                 unchanged. Technically this doesn't fit the assumptions made by
                 privacy accounting mechanism, but it can be a good approximation when
                 using Poisson sampling is unfeasible.
-            clipping: Per sample gradient clipping mechanism ("flat" or "per_layer").
+            clipping: Per sample gradient clipping mechanism ("flat" or "per_layer" or "adaptive").
                 Flat clipping calculates the norm of the entire gradient over
-                all parameters, while per layer clipping sets individual norms for
-                every parameter tensor. Flat clipping is usually preferred, but using
-                per layer clipping in combination with distributed training can provide
-                notable performance gains.
+                all parameters, per layer clipping sets individual norms for
+                every parameter tensor, and adaptive clipping updates clipping bound per iteration.
+                Flat clipping is usually preferred, but using per layer clipping in combination
+                with distributed training can provide notable performance gains.
             noise_generator: torch.Generator() object used as a source of randomness for
                 the noise
 
@@ -357,6 +360,21 @@ class PrivacyEngine:
         if noise_generator and self.secure_mode:
             raise ValueError("Passing seed is prohibited in secure mode")
 
+        # compare module parameter with optimizer parameters
+        if not all(
+            torch.eq(i, j).all()
+            for i, j in zip(
+                list(module.parameters()),
+                sum(
+                    [param_group["params"] for param_group in optimizer.param_groups],
+                    [],
+                ),
+            )
+        ):
+            raise ValueError(
+                "Module parameters are different than optimizer Parameters"
+            )
+
         distributed = isinstance(module, (DPDDP, DDP))
 
         module = self._prepare_model(
@@ -439,10 +457,10 @@ class PrivacyEngine:
                 using Poisson sampling is unfeasible.
             clipping: Per sample gradient clipping mechanism ("flat" or "per_layer").
                 Flat clipping calculates the norm of the entire gradient over
-                all parameters, while per layer clipping sets individual norms for
-                every parameter tensor. Flat clipping is usually preferred, but using
-                per layer clipping in combination with distributed training can provide
-                notable performance gains.
+                all parameters, per layer clipping sets individual norms for
+                every parameter tensor, and adaptive clipping updates clipping bound per iteration.
+                Flat clipping is usually preferred, but using per layer clipping in combination
+                with distributed training can provide notable performance gains.
 
         Returns:
             Tuple of (model, optimizer, data_loader).
@@ -493,3 +511,68 @@ class PrivacyEngine:
             Privacy budget (epsilon) expended so far.
         """
         return self.accountant.get_epsilon(delta)
+
+    def save_checkpoint(
+        self,
+        *,
+        path: Union[str, os.PathLike, BinaryIO, IO[bytes]],
+        module: GradSampleModule,
+        optimizer: Optional[DPOptimizer] = None,
+        noise_scheduler: Optional[_NoiseScheduler] = None,
+        checkpoint_dict: Optional[Dict[str, Any]] = None,
+        module_state_dict_kwargs: Optional[Dict[str, Any]] = None,
+        torch_save_kwargs: Optional[Dict[str, Any]] = None,
+    ):
+        """
+        Saves the state_dict of module, optimzer, and accountant at path.
+        Args:
+            path: Path to save the state dict objects.
+            module: GradSampleModule to save; wrapped module's state_dict is saved.
+            optimizer: DPOptimizer to save; wrapped optimizer's state_dict is saved.
+            module_state_dict_kwargs: dict of kwargs to pass to ``module.state_dict()``
+            torch_save_kwargs: dict of kwargs to pass to ``torch.save()``
+
+        """
+        checkpoint_dict = checkpoint_dict or {}
+        checkpoint_dict["module_state_dict"] = module.state_dict(
+            **(module_state_dict_kwargs or {})
+        )
+        checkpoint_dict["privacy_accountant_state_dict"] = self.accountant.state_dict()
+        if optimizer is not None:
+            checkpoint_dict["optimizer_state_dict"] = optimizer.state_dict()
+        if noise_scheduler is not None:
+            checkpoint_dict["noise_scheduler_state_dict"] = noise_scheduler.state_dict()
+
+        torch.save(checkpoint_dict, path, **(torch_save_kwargs or {}))
+
+    def load_checkpoint(
+        self,
+        *,
+        path: Union[str, os.PathLike, BinaryIO, IO[bytes]],
+        module: GradSampleModule,
+        optimizer: Optional[DPOptimizer] = None,
+        noise_scheduler: Optional[_NoiseScheduler] = None,
+        module_load_dict_kwargs: Optional[Dict[str, Any]] = None,
+        torch_load_kwargs: Optional[Dict[str, Any]] = None,
+    ) -> Dict:
+        checkpoint = torch.load(path, **(torch_load_kwargs or {}))
+        module.load_state_dict(
+            checkpoint["module_state_dict"], **(module_load_dict_kwargs or {})
+        )
+        self.accountant.load_state_dict(checkpoint["privacy_accountant_state_dict"])
+
+        optimizer_state_dict = checkpoint.pop("optimizer_state_dict", {})
+        if optimizer is not None and len(optimizer_state_dict) > 0:
+            optimizer.load_state_dict(optimizer_state_dict)
+        elif (optimizer is not None) ^ (len(optimizer_state_dict) > 0):
+            # warn if only one of them is available
+            warnings.warn(
+                f"optimizer_state_dict has {len(optimizer_state_dict)} items"
+                f" but optimizer is {'' if optimizer else 'not'} provided."
+            )
+
+        noise_scheduler_state_dict = checkpoint.pop("noise_scheduler_state_dict", {})
+        if noise_scheduler is not None and len(noise_scheduler_state_dict) > 0:
+            noise_scheduler.load_state_dict(noise_scheduler_state_dict)
+
+        return checkpoint
