@@ -14,6 +14,7 @@
 # limitations under the License.
 
 import abc
+import io
 import math
 import unittest
 from abc import ABC
@@ -28,8 +29,11 @@ from hypothesis import given, settings
 from opacus import PrivacyEngine
 from opacus.layers.dp_multihead_attention import DPMultiheadAttention
 from opacus.optimizers.optimizer import _generate_noise
+from opacus.scheduler import StepNoise, _NoiseScheduler
 from opacus.utils.module_utils import are_state_dict_equal
 from opacus.validators.errors import UnsupportedModuleError
+from opacus.validators.module_validator import ModuleValidator
+from opt_einsum import contract
 from torch.utils.data import DataLoader, Dataset, TensorDataset
 from torchvision import models, transforms
 from torchvision.datasets import FakeData
@@ -45,7 +49,7 @@ def get_grad_sample_aggregated(tensor: torch.Tensor, loss_type: str = "mean"):
     if loss_type not in ("sum", "mean"):
         raise ValueError(f"loss_type = {loss_type}. Only 'sum' and 'mean' supported")
 
-    grad_sample_aggregated = torch.einsum("i...->...", tensor.grad_sample)
+    grad_sample_aggregated = contract("i...->...", tensor.grad_sample)
     if loss_type == "mean":
         b_sz = tensor.grad_sample.shape[0]
         grad_sample_aggregated /= b_sz
@@ -462,12 +466,142 @@ class BasePrivacyEngineTest(ABC):
                 "Model parameters after deterministic run must match",
             )
 
+    def test_param_equal_module_optimizer(self):
+        """Test that the privacy engine raises error if nn.Module parameters are not equal to optimizer parameters"""
+        model = models.densenet121(pretrained=True)
+        num_ftrs = model.classifier.in_features
+        model.classifier = nn.Sequential(nn.Linear(num_ftrs, 10), nn.Sigmoid())
+        optimizer = torch.optim.SGD(
+            model.parameters(), lr=0.01, momentum=0, weight_decay=0
+        )
+        dl = self._init_data()
+        model = ModuleValidator.fix(model)
+        privacy_engine = PrivacyEngine()
+        with self.assertRaisesRegex(
+            ValueError, "Module parameters are different than optimizer Parameters"
+        ):
+            _, _, _ = privacy_engine.make_private(
+                module=model,
+                optimizer=optimizer,
+                data_loader=dl,
+                noise_multiplier=1.1,
+                max_grad_norm=1.0,
+            )
+
+        # if optimizer is defined after ModuleValidator.fix() then raise no error
+        optimizer = torch.optim.SGD(
+            model.parameters(), lr=0.01, momentum=0, weight_decay=0
+        )
+        _, _, _ = privacy_engine.make_private(
+            module=model,
+            optimizer=optimizer,
+            data_loader=dl,
+            noise_multiplier=1.1,
+            max_grad_norm=1.0,
+        )
+        self.assertTrue(1, 1)
+
+    @given(noise_scheduler=st.sampled_from([None, StepNoise]))
+    @settings(deadline=None)
+    def test_checkpoints(self, noise_scheduler: Optional[_NoiseScheduler]):
+        # 1. Disable poisson sampling to avoid randomness in data loading caused by changing seeds.
+        # 2. Use noise_multiplier=0.0 to avoid randomness in torch.normal()
+        # create a set of components: set 1
+        torch.manual_seed(1)
+        m1, opt1, dl1, pe1 = self._init_private_training(
+            noise_multiplier=0.0, poisson_sampling=False
+        )
+        s1 = (
+            noise_scheduler(optimizer=opt1, step_size=1, gamma=1.0)
+            if noise_scheduler is not None
+            else None
+        )
+        # create a different set of components: set 2
+        torch.manual_seed(2)
+        m2, opt2, _, pe2 = self._init_private_training(
+            noise_multiplier=2.0, poisson_sampling=False
+        )
+        s2 = (
+            noise_scheduler(optimizer=opt2, step_size=1, gamma=2.0)
+            if noise_scheduler is not None
+            else None
+        )
+
+        # check that two sets of components are different
+        self.assertFalse(are_state_dict_equal(m1.state_dict(), m2.state_dict()))
+        if noise_scheduler:
+            self.assertNotEqual(s1.state_dict(), s2.state_dict())
+        self.assertNotEqual(opt1.noise_multiplier, opt2.noise_multiplier)
+
+        # train set 1 for a few steps
+        self._train_steps(m1, opt1, dl1)
+        if noise_scheduler:
+            s1.step()
+
+        # load into set 2
+        checkpoint_to_save = {"foo": "bar"}
+        with io.BytesIO() as bytesio:
+            pe1.save_checkpoint(
+                path=bytesio,
+                module=m1,
+                optimizer=opt1,
+                noise_scheduler=s1,
+                checkpoint_dict=checkpoint_to_save,
+            )
+            bytesio.seek(0)
+            loaded_checkpoint = pe2.load_checkpoint(
+                path=bytesio, module=m2, optimizer=opt2, noise_scheduler=s2
+            )
+
+        # check if loaded checkpoint has dummy dict
+        self.assertTrue(
+            "foo" in loaded_checkpoint and loaded_checkpoint["foo"] == "bar"
+        )
+        # check the two sets of components are now the same
+        self.assertEqual(pe1.accountant.state_dict(), pe2.accountant.state_dict())
+        self.assertTrue(are_state_dict_equal(m1.state_dict(), m2.state_dict()))
+        if noise_scheduler:
+            self.assertEqual(s1.state_dict(), s2.state_dict())
+        # check that non-state params are still different
+        self.assertNotEqual(opt1.noise_multiplier, opt2.noise_multiplier)
+
+        # train the now loaded set 2 some more (change noise multiplier before doing so)
+        opt2.noise_multiplier = 0.0
+        self._train_steps(m2, opt2, dl1)
+        if noise_scheduler:
+            s2.step()
+
+        # recreate set 1 from scratch (set11) and check it is different from the trained set 2
+        torch.manual_seed(1)
+        m11, opt11, dl11, _ = self._init_private_training(
+            noise_multiplier=0.0, poisson_sampling=False
+        )
+        s11 = (
+            noise_scheduler(optimizer=opt11, step_size=1, gamma=1.0)
+            if noise_scheduler is not None
+            else None
+        )
+        self.assertFalse(are_state_dict_equal(m2.state_dict(), m11.state_dict()))
+        if noise_scheduler:
+            self.assertNotEqual(s2.state_dict(), s11.state_dict())
+        # train the recreated set for the same number of steps
+        self._train_steps(m11, opt11, dl11)
+        if noise_scheduler:
+            s11.step()
+        self._train_steps(m11, opt11, dl11)
+        if noise_scheduler:
+            s11.step()
+        # check that recreated set is now same as the original set 1 after training
+        self.assertTrue(are_state_dict_equal(m2.state_dict(), m11.state_dict()))
+        if noise_scheduler:
+            self.assertEqual(s2.state_dict(), s11.state_dict())
+
     @given(
         noise_multiplier=st.floats(0.5, 5.0),
         max_steps=st.integers(8, 10),
-        secure_mode=st.booleans(),
+        secure_mode=st.just(False),  # TODO: enable after fixing torchcsprng build
     )
-    @settings(max_examples=20, deadline=None)
+    @settings(deadline=None)
     def test_noise_level(
         self, noise_multiplier: float, max_steps: int, secure_mode: bool
     ):
@@ -523,6 +657,7 @@ class BasePrivacyEngineTest(ABC):
             secure_mode=secure_mode,
         )
 
+    @unittest.skip("requires torchcsprng compatible with new pytorch versions")
     @patch("torch.normal", MagicMock(return_value=torch.Tensor([0.6])))
     def test_generate_noise_in_secure_mode(self):
         """
