@@ -17,7 +17,7 @@ import io
 import logging
 import sys
 from collections import OrderedDict
-from typing import Iterable, List
+from typing import Iterable, List, Tuple
 
 import torch
 import torch.nn as nn
@@ -44,7 +44,7 @@ def parametrized_modules(module: nn.Module) -> Iterable[nn.Module]:
     )
 
 
-def trainable_modules(module: nn.Module) -> Iterable[nn.Module]:
+def trainable_modules(module: nn.Module) -> Iterable[Tuple[str, nn.Module]]:
     """
     Recursively iterates over all submodules, returning those that
     have parameters and are trainable (ie they want a grad).
@@ -56,6 +56,16 @@ def trainable_modules(module: nn.Module) -> Iterable[nn.Module]:
     )
 
 
+def trainable_parameters(module: nn.Module) -> Iterable[Tuple[str, nn.Parameter]]:
+    """
+    Recursively iterates over all parameters, returning those that
+    are trainable (ie they want a grad).
+    """
+    yield from (
+        (p_name, p) for (p_name, p) in module.named_parameters() if p.requires_grad
+    )
+
+
 def requires_grad(module: nn.Module, *, recurse: bool = False) -> bool:
     """
     Checks if any parameters in a specified module require gradients.
